@@ -65,6 +65,16 @@ powervm_opts = [
                help="Name of the PowerVM virtual switch to be used when "
                     "mapping Linux based network ports to PowerVM virtual "
                     "Ethernet devices"),
+    cfg.BoolOpt('remove_vopt_media_on_boot',
+                default=False,
+                help="If enabled, tells the PowerVM driver to trigger the "
+                     "removal of the media from the virtual optical device "
+                     "used for initialization of VMs on spawn after "
+                     "'remove_vopt_media_time' minutes."),
+    cfg.IntOpt('remove_vopt_media_time',
+               default=60, min=0,
+               help="The amount of time in minutes after a VM has been "
+                    "created for the virtual optical media to be removed."),
     cfg.BoolOpt('use_rmc_mgmt_vif',
                 default=True,
                 help="If enabled, tells the PowerVM Driver to create an RMC "
@@ -88,20 +98,10 @@ powervm_opts = [
 localdisk_opts = [
     cfg.StrOpt('volume_group_name',
                default='',
-               help='Volume Group to use for block device operations.  Must '
-                    'not be rootvg.  If disk_driver is localdisk, and more '
-                    'than one non-rootvg volume group exists across the '
-                    'Virtual I/O Servers, then this attribute must be '
-                    'specified.'),
-    cfg.StrOpt('volume_group_vios_name',
-               default='',
-               help='(Optional) The name of the Virtual I/O Server hosting '
-                    'the volume group.  If this is not specified, the system '
-                    'will query through the Virtual I/O Servers looking for '
-                    'one that matches the volume_group_vios_name.  This is '
-                    'only needed if the system has multiple Virtual I/O '
-                    'Servers with a non-rootvg volume group whose name is '
-                    'duplicated.')
+               help='Volume Group to use for block device operations.  If '
+                    'disk_driver is localdisk, then this attribute must be '
+                    'specified.  It is strongly recommended NOT to use '
+                    'rootvg.'),
 ]
 
 ssp_opts = [
@@ -116,7 +116,7 @@ ssp_opts = [
 vol_adapter_opts = [
     cfg.StrOpt('fc_attach_strategy',
                choices=['vscsi', 'npiv'], ignore_case=True,
-               default='vscsi',
+               default='vscsi', mutable=True,
                help='The Fibre Channel Volume Strategy defines how FC Cinder '
                     'volumes should be attached to the Virtual Machine.  The '
                     'options are: npiv or vscsi. If npiv is selected then '
@@ -195,7 +195,10 @@ swift_opts = [
     cfg.StrOpt('swift_auth_url', help='The Keystone authorization url. '
                'Example: "http://keystone-hostname:5000/v3"'),
     cfg.StrOpt('swift_cacert', required=False, help='Path to CA certificate '
-               'file.  Example: /etc/swiftclient/myca.pem')
+               'file.  Example: /etc/swiftclient/myca.pem'),
+    cfg.StrOpt('swift_endpoint_type', help='The endpoint/interface type for '
+               'the Swift client to select from the Keystone Service Catalog '
+               'for the connection URL.  Swift defaults to "publicURL".')
 ]
 
 vnc_opts = [
