@@ -1,4 +1,4 @@
-# Copyright 2015, 2016 IBM Corp.
+# Copyright 2015, 2017 IBM Corp.
 #
 # All Rights Reserved.
 #
@@ -20,13 +20,10 @@ from __future__ import absolute_import
 import fixtures
 import mock
 
-from nova.virt.powervm import driver
+from nova.virt.powervm_ext import driver
 
 from nova.virt import fake
 from pypowervm.tests import test_fixtures as pvm_fx
-from pypowervm.tests.test_utils import pvmhttp
-
-MS_HTTPRESP_FILE = "fake_managedsystem.txt"
 
 FAKE_INST_UUID = 'b6513403-fd7f-4ad0-ab27-f73bacbd3929'
 FAKE_INST_UUID_PVM = '36513403-FD7F-4AD0-AB27-F73BACBD3929'
@@ -96,11 +93,17 @@ class PowerVMComputeDriver(fixtures.Fixture):
     @mock.patch('nova_powervm.virt.powervm.disk.localdisk.LocalStorage')
     @mock.patch('nova_powervm.virt.powervm.driver.PowerVMDriver._get_adapter')
     @mock.patch('pypowervm.tasks.partition.get_this_partition')
+    @mock.patch('pypowervm.tasks.cna.find_orphaned_trunks')
     def _init_host(self, *args):
-        ms_http = pvmhttp.load_pvm_resp(
-            MS_HTTPRESP_FILE, adapter=mock.Mock()).get_response()
-        # Pretend it just returned one host
-        ms_http.feed.entries = [ms_http.feed.entries[0]]
+
+        self.mock_sys = self.useFixture(fixtures.MockPatch(
+            'pypowervm.wrappers.managed_system.System.get')).mock
+        self.mock_sys.return_value = [mock.Mock(
+            uuid='host_uuid',
+            system_name='Server-8247-21L-SN9999999',
+            proc_compat_modes=('default', 'POWER7', 'POWER8'),
+            migration_data={'active_migrations_supported': 16,
+                            'active_migrations_in_progress': 0})]
 
         # Mock active vios
         self.get_active_vios = self.useFixture(fixtures.MockPatch(
@@ -110,7 +113,6 @@ class PowerVMComputeDriver(fixtures.Fixture):
         self.useFixture(fixtures.MockPatch(
             'pypowervm.tasks.partition.validate_vios_ready'))
 
-        self.drv.adapter.read.return_value = ms_http
         self.drv.session = self.drv.adapter.session
         self.drv.init_host('FakeHost')
 
@@ -182,9 +184,7 @@ class TaskFlow(fixtures.Fixture):
                 testcase.assertEqual(expected, observed.name)
 
         # Compare the list of expected against added.
-        for func in map(compare_tasks, expected_tasks, self.tasks_added):
-            # Call the function to do the compare
-            func
+        map(compare_tasks, expected_tasks, self.tasks_added)
 
 
 class DriverTaskFlow(TaskFlow):
@@ -192,4 +192,4 @@ class DriverTaskFlow(TaskFlow):
     def __init__(self):
         super(DriverTaskFlow, self).__init__(
             linear_flow='nova_powervm.virt.powervm.driver.tf_lf',
-            engines='nova_powervm.virt.powervm.driver.tf_eng')
+            engines='nova_powervm.virt.powervm.tasks.base.tf_eng')
