@@ -74,9 +74,10 @@ class TestNPIVAdapter(test_vol.TestVolumeAdapter):
         @mock.patch('pypowervm.wrappers.virtual_io_server.VIOS.getter')
         @mock.patch('nova_powervm.virt.powervm.vm.get_pvm_uuid')
         def init_vol_adpt(mock_pvm_uuid, mock_getter):
-            con_info = {'data': {'initiator_target_map': {'i1': ['t1'],
+            con_info = {'serial': 'id',
+                        'data': {'initiator_target_map': {'i1': ['t1'],
                                                           'i2': ['t2', 't3']},
-                        'target_lun': '1', 'volume_id': 'id'}}
+                                 'target_lun': '1'}}
             mock_inst = mock.MagicMock()
             mock_pvm_uuid.return_value = '1234'
 
@@ -628,8 +629,7 @@ class TestNPIVAdapter(test_vol.TestVolumeAdapter):
         # Verify that on migration, the WWPNs are reversed.
         self.assertEqual(2, self.vol_drv.stg_ftsk.feed.reverse.call_count)
 
-    @mock.patch('pypowervm.tasks.vfc_mapper.'
-                'build_migration_mappings')
+    @mock.patch('pypowervm.tasks.vfc_mapper.build_migration_mappings')
     @mock.patch('nova_powervm.virt.powervm.volume.npiv.NPIVVolumeAdapter.'
                 '_fabric_names')
     def test_pre_live_migration_on_destination(
@@ -648,8 +648,7 @@ class TestNPIVAdapter(test_vol.TestVolumeAdapter):
 
         # Order of the mappings is not important.
         self.assertEqual(
-            set({'b', 'a'}),
-            set(jsonutils.loads(mig_data.get('vfc_lpm_mappings'))))
+            {'b', 'a'}, set(jsonutils.loads(mig_data.get('vfc_lpm_mappings'))))
 
     def test_set_fabric_meta(self):
         port_map = [('1', 'aa AA'), ('2', 'bb BB'),
