@@ -1,4 +1,4 @@
-# Copyright 2015, 2016 IBM Corp.
+# Copyright 2015, 2017 IBM Corp.
 #
 # All Rights Reserved.
 #
@@ -21,20 +21,18 @@ from nova import utils
 
 from oslo_log import log as logging
 from pypowervm.wrappers import network as pvm_net
+from taskflow import task
 
 from nova_powervm import conf as cfg
-from nova_powervm.virt.powervm.i18n import _LE
-from nova_powervm.virt.powervm.i18n import _LI
-from nova_powervm.virt.powervm.i18n import _LW
-from nova_powervm.virt.powervm.tasks import base as pvm_task
 from nova_powervm.virt.powervm import vif
 from nova_powervm.virt.powervm import vm
 
+
 LOG = logging.getLogger(__name__)
 CONF = cfg.CONF
 
 
-class UnplugVifs(pvm_task.PowerVMTask):
+class UnplugVifs(task.Task):
 
     """The task to unplug Virtual Network Interfaces from a VM."""
 
@@ -54,20 +52,18 @@ class UnplugVifs(pvm_task.PowerVMTask):
         self.network_infos = network_infos or []
         self.host_uuid = host_uuid
         self.slot_mgr = slot_mgr
+        self.instance = instance
 
-        super(UnplugVifs, self).__init__(instance, 'unplug_vifs',
-                                         requires=['lpar_wrap'])
+        super(UnplugVifs, self).__init__('unplug_vifs', requires=['lpar_wrap'])
 
-    def execute_impl(self, lpar_wrap):
+    def execute(self, lpar_wrap):
         # If the state is not in an OK state for deleting, then throw an
         # error up front.
         modifiable, reason = lpar_wrap.can_modify_io()
         if not modifiable:
-            LOG.error(_LE('Unable to remove VIFs from instance %(inst)s '
-                          'because the system is not in a correct state.  '
-                          'The reason reported by the system is: %(reason)s'),
-                      {'inst': self.instance.name, 'reason': reason},
-                      instance=self.instance)
+            LOG.error("Unable to remove VIFs in the instance's current state. "
+                      "The reason reported by the system is: %(reason)s",
+                      {'reason': reason}, instance=self.instance)
             raise exception.VirtualInterfaceUnplugException(reason=reason)
 
         # Get all the current Client Network Adapters (CNA) on the VM itself.
@@ -81,7 +77,7 @@ class UnplugVifs(pvm_task.PowerVMTask):
         return cna_w_list
 
 
-class PlugVifs(pvm_task.PowerVMTask):
+class PlugVifs(task.Task):
 
     """The task to plug the Virtual Network Interfaces to a VM."""
 
@@ -109,9 +105,10 @@ class PlugVifs(pvm_task.PowerVMTask):
         self.slot_mgr = slot_mgr
         self.crt_network_infos, self.update_network_infos = [], []
         self.cnas, self.vnics = None, None
+        self.instance = instance
 
-        super(PlugVifs, self).__init__(
-            instance, 'plug_vifs', provides='vm_cnas', requires=['lpar_wrap'])
+        super(PlugVifs, self).__init__('plug_vifs', provides='vm_cnas',
+                                       requires=['lpar_wrap'])
 
     def _vif_exists(self, network_info):
         """Does the instance have a CNA/VNIC (as appropriate) for a given net?
@@ -135,7 +132,7 @@ class PlugVifs(pvm_task.PowerVMTask):
 
         return network_info['address'] in [vm.norm_mac(v.mac) for v in vifs]
 
-    def execute_impl(self, lpar_wrap):
+    def execute(self, lpar_wrap):
         # We will have two types of network infos.  One is for newly created
         # vifs.  The others are those that exist, but should be re-'treated'
         for network_info in self.network_infos:
@@ -151,12 +148,9 @@ class PlugVifs(pvm_task.PowerVMTask):
         # Check to see if the LPAR is OK to add VIFs to.
         modifiable, reason = lpar_wrap.can_modify_io()
         if not modifiable and self.crt_network_infos:
-            LOG.error(_LE('Unable to create VIF(s) for instance %(sys)s.  The '
-                          'VM was in a state where VIF plugging is not '
-                          'acceptable.  The reason from the system is: '
-                          '%(reason)s'),
-                      {'sys': self.instance.name, 'reason': reason},
-                      instance=self.instance)
+            LOG.error("Unable to create VIF(s) in the instance's current "
+                      "state. The reason reported by the system is: "
+                      "%(reason)s", {'reason': reason}, instance=self.instance)
             raise exception.VirtualInterfaceCreateException()
 
         # TODO(KYLEH): We're setting up to wait for an instance event.  The
@@ -166,8 +160,8 @@ class PlugVifs(pvm_task.PowerVMTask):
         # See: https://bugs.launchpad.net/nova/+bug/1535918
         undo_host_change = False
         if self.instance.host != CONF.host:
-            LOG.warning(_LW('Instance was not assigned to this host. '
-                            'It was assigned to: %s'), self.instance.host,
+            LOG.warning('Instance was not assigned to this host. '
+                        'It was assigned to: %s', self.instance.host,
                         instance=self.instance)
             # Update the instance...
             old_host = self.instance.host
@@ -179,9 +173,8 @@ class PlugVifs(pvm_task.PowerVMTask):
         # not wait for the neutron event as that likely won't be sent (it was
         # already done).
         for network_info in self.update_network_infos:
-            LOG.info(_LI("Updating VIF with mac %(mac)s for instance %(sys)s"),
-                     {'mac': network_info['address'],
-                      'sys': self.instance.name}, instance=self.instance)
+            LOG.info("Updating VIF with mac %(mac)s",
+                     {'mac': network_info['address']}, instance=self.instance)
             vif.plug(self.adapter, self.host_uuid, self.instance,
                      network_info, self.slot_mgr, new_vif=False)
 
@@ -192,10 +185,8 @@ class PlugVifs(pvm_task.PowerVMTask):
                     deadline=CONF.vif_plugging_timeout,
                     error_callback=self._vif_callback_failed):
                 for network_info in self.crt_network_infos:
-                    LOG.info(_LI('Creating VIF with mac %(mac)s for instance '
-                                 '%(sys)s'),
-                             {'mac': network_info['address'],
-                              'sys': self.instance.name},
+                    LOG.info('Creating VIF with mac %(mac)s.',
+                             {'mac': network_info['address']},
                              instance=self.instance)
                     new_vif = vif.plug(
                         self.adapter, self.host_uuid, self.instance,
@@ -204,13 +195,12 @@ class PlugVifs(pvm_task.PowerVMTask):
                                                             pvm_net.CNA):
                         self.cnas.append(new_vif)
         except eventlet.timeout.Timeout:
-            LOG.error(_LE('Error waiting for VIF to be created for instance '
-                          '%(sys)s'), {'sys': self.instance.name},
+            LOG.error('Error waiting for VIF to be created.',
                       instance=self.instance)
             raise exception.VirtualInterfaceCreateException()
         finally:
             if undo_host_change:
-                LOG.info(_LI('Undoing temporary host assignment to instance.'),
+                LOG.info('Undoing temporary host assignment to instance.',
                          instance=self.instance)
                 self.instance.host = old_host
                 self.instance.save()
@@ -218,9 +208,8 @@ class PlugVifs(pvm_task.PowerVMTask):
         return self.cnas
 
     def _vif_callback_failed(self, event_name, instance):
-        LOG.error(_LE('VIF Plug failure for callback on event '
-                      '%(event)s for instance %(uuid)s'),
-                  {'event': event_name, 'uuid': instance.uuid})
+        LOG.error('VIF plug failure for callback on event %(event)s.',
+                  {'event': event_name}, instance=instance)
         if CONF.vif_plugging_is_fatal:
             raise exception.VirtualInterfaceCreateException()
 
@@ -243,14 +232,13 @@ class PlugVifs(pvm_task.PowerVMTask):
         else:
             return []
 
-    def revert_impl(self, lpar_wrap, result, flow_failures):
+    def revert(self, lpar_wrap, result, flow_failures):
         if not self.network_infos:
             return
 
         # The parameters have to match the execute method, plus the response +
         # failures even if only a subset are used.
-        LOG.warning(_LW('VIF creation being rolled back for instance '
-                        '%(inst)s'), {'inst': self.instance.name},
+        LOG.warning('VIF creation is being rolled back.',
                     instance=self.instance)
 
         # Get the current adapters on the system
@@ -259,14 +247,12 @@ class PlugVifs(pvm_task.PowerVMTask):
             try:
                 vif.unplug(self.adapter, self.host_uuid, self.instance,
                            network_info, self.slot_mgr, cna_w_list=cna_w_list)
-            except Exception as e:
-                LOG.exception(e)
-                LOG.warning(_LW("An exception occurred during an unplug "
-                                "in the vif rollback.  Ignoring."),
-                            instance=self.instance)
+            except Exception:
+                LOG.exception("Error unplugging during vif rollback. "
+                              "Ignoring.", instance=self.instance)
 
 
-class PlugMgmtVif(pvm_task.PowerVMTask):
+class PlugMgmtVif(task.Task):
 
     """The task to plug the Management VIF into a VM."""
 
@@ -288,30 +274,29 @@ class PlugMgmtVif(pvm_task.PowerVMTask):
         self.adapter = adapter
         self.host_uuid = host_uuid
         self.slot_mgr = slot_mgr
+        self.instance = instance
 
-        super(PlugMgmtVif, self).__init__(
-            instance, 'plug_mgmt_vif', provides='mgmt_cna',
-            requires=['vm_cnas'])
+        super(PlugMgmtVif, self).__init__('plug_mgmt_vif', provides='mgmt_cna',
+                                          requires=['vm_cnas'])
 
-    def execute_impl(self, vm_cnas):
+    def execute(self, vm_cnas):
         # If configured to not use RMC mgmt vifs, then return None.  Need to
         # return None because the Config Drive step (which may be used...may
         # not be) required the mgmt vif.
         if not CONF.powervm.use_rmc_mgmt_vif:
-            LOG.debug('No management VIF created for instance %s as the conf '
-                      'option use_rmc_mgmt_vif is set to False',
-                      self.instance.name)
+            LOG.debug('No management VIF created because '
+                      'CONF.powervm.use_rmc_mgmt_vif is False',
+                      instance=self.instance)
             return None
 
-        LOG.info(_LI('Plugging the Management Network Interface to instance '
-                     '%s'), self.instance.name, instance=self.instance)
+        LOG.info('Plugging the management network interface.',
+                 instance=self.instance)
         # Determine if we need to create the secure RMC VIF.  This should only
         # be needed if there is not a VIF on the secure RMC vSwitch
         vswitch = vif.get_secure_rmc_vswitch(self.adapter, self.host_uuid)
         if vswitch is None:
-            LOG.warning(_LW('No management VIF created for instance %s due to '
-                        'lack of Management Virtual Switch'),
-                        self.instance.name)
+            LOG.warning('No management VIF created due to lack of management '
+                        'virtual switch', instance=self.instance)
             return None
 
         # This next check verifies that there are no existing NICs on the
@@ -323,8 +308,7 @@ class PlugMgmtVif(pvm_task.PowerVMTask):
             has_mgmt_vif = vswitch.href in [cna.vswitch_uri for cna in vm_cnas]
 
         if has_mgmt_vif:
-            LOG.debug('Management VIF already created for instance %s',
-                      self.instance.name)
+            LOG.debug('Management VIF already exists.', instance=self.instance)
             return None
 
         # Return the created management CNA
